@@ -28,6 +28,9 @@
 #endif
 
 extern int curcolumns;
+#if	FD >= 3
+extern int widedigit;
+#endif
 extern int mark;
 extern off_t marksize;
 extern off_t blocksize;
@@ -53,7 +56,7 @@ extern int hideclock;
 extern int fdmode;
 #endif
 
-static VOID NEAR replacefname __P_((char *));
+static int NEAR dochdir4 __P_((CONST char *, int));
 static int cur_up __P_((CONST char *));
 static int cur_down __P_((CONST char *));
 static int cur_right __P_((CONST char *));
@@ -74,7 +77,7 @@ static int mark_file2 __P_((CONST char *));
 static int mark_file3 __P_((CONST char *));
 static int mark_all __P_((CONST char *));
 static int mark_reverse __P_((CONST char *));
-static reg_t *NEAR prepareregexp __P_((CONST char *, CONST char *));
+static reg_ex_t *NEAR prepareregexp __P_((CONST char *, CONST char *));
 static int mark_find __P_((CONST char *));
 static int in_dir __P_((CONST char *));
 static int out_dir __P_((CONST char *));
@@ -161,7 +164,7 @@ static int no_operation __P_((CONST char *));
 
 #include "functabl.h"
 
-reg_t *findregexp = NULL;
+reg_ex_t *findregexp = NULL;
 #ifndef	_NOWRITEFS
 int writefs = 0;
 #endif
@@ -170,7 +173,7 @@ int loopcursor = 0;
 #endif
 int internal_status = FNC_FAIL;
 int maxbind = 0;
-#ifndef	_NOCUSTOMIZE
+#if	defined (DEP_DYNAMICLIST) || !defined (_NOCUSTOMIZE)
 int origmaxbind = 0;
 #endif
 #ifdef	DEP_DYNAMICLIST
@@ -328,15 +331,16 @@ bindlist_t bindlist
 };
 
 
-static VOID NEAR replacefname(name)
-char *name;
+static int NEAR dochdir4(path, raw)
+CONST char *path;
+int raw;
 {
-	if (filepos < maxfile) free2(filelist[filepos].name);
-	else for (maxfile = 0; maxfile < filepos; maxfile++)
-		if (!filelist[maxfile].name)
-			filelist[maxfile].name = strdup2(nullstr);
-	filelist[filepos].name = (name) ? name : strdup2(parentpath);
-	filelist[filepos].tmpflags |= F_ISCHGDIR;
+	if (chdir4(path, raw, archivefile) < 0) {
+		warning(-1, path);
+		return(FNC_CANCEL);
+	}
+
+	return(FNC_EFFECT);
 }
 
 static int cur_up(arg)
@@ -348,7 +352,7 @@ CONST char *arg;
 	int n, old;
 
 	old = filepos;
-	if (!arg || (n = atoi2(arg)) <= 0) n = 1;
+	if (!arg || (n = Xatoi(arg)) <= 0) n = 1;
 	filepos -= n;
 #if	FD >= 2
 	if (loopcursor) {
@@ -373,7 +377,7 @@ CONST char *arg;
 	int n, old;
 
 	old = filepos;
-	if (!arg || (n = atoi2(arg)) <= 0) n = 1;
+	if (!arg || (n = Xatoi(arg)) <= 0) n = 1;
 	filepos += n;
 #if	FD >= 2
 	if (loopcursor) {
@@ -398,7 +402,7 @@ CONST char *arg;
 	int n, r, p, old;
 
 	old = filepos;
-	if (!arg || (n = atoi2(arg)) <= 0) n = 1;
+	if (!arg || (n = Xatoi(arg)) <= 0) n = 1;
 	r = FILEPERROW;
 	p = FILEPERPAGE;
 	filepos += r * n;
@@ -438,7 +442,7 @@ CONST char *arg;
 	int n, r, old;
 
 	old = filepos;
-	if (!arg || (n = atoi2(arg)) <= 0) n = 1;
+	if (!arg || (n = Xatoi(arg)) <= 0) n = 1;
 	r = FILEPERROW;
 	filepos -= r * n;
 
@@ -466,7 +470,7 @@ CONST char *arg;
 	int n, p, old;
 
 	old = filepos;
-	if (!arg || (n = atoi2(arg)) <= 0) n = 1;
+	if (!arg || (n = Xatoi(arg)) <= 0) n = 1;
 	p = FILEPERPAGE;
 	filepos = (filepos / p + n) * p;
 	if (filepos >= maxfile) {
@@ -483,7 +487,7 @@ CONST char *arg;
 	int n, p, old;
 
 	old = filepos;
-	if (!arg || (n = atoi2(arg)) <= 0) n = 1;
+	if (!arg || (n = Xatoi(arg)) <= 0) n = 1;
 	p = FILEPERPAGE;
 	filepos = (filepos / p - n) * p;
 	if (filepos < 0) {
@@ -498,7 +502,7 @@ CONST char *arg;
 static int cur_top(arg)
 CONST char *arg;
 {
-	if (filepos == 0) return(FNC_NONE);
+	if (filepos <= 0) return(FNC_NONE);
 	filepos = 0;
 
 	return(FNC_UPDATE);
@@ -508,7 +512,7 @@ CONST char *arg;
 static int cur_bottom(arg)
 CONST char *arg;
 {
-	if (filepos == maxfile - 1) return(FNC_NONE);
+	if (filepos >= maxfile - 1) return(FNC_NONE);
 	filepos = maxfile - 1;
 
 	return(FNC_UPDATE);
@@ -544,7 +548,7 @@ CONST char *arg;
 			else {
 				i = strlen(filelist[filepos].linkname);
 				if (i > strsize(tmp)) i = strsize(tmp);
-				strncpy2(tmp, filelist[filepos].linkname, i);
+				Xstrncpy(tmp, filelist[filepos].linkname, i);
 			}
 		}
 		else
@@ -612,18 +616,18 @@ static VOID NEAR markcount(VOID_A)
 #ifndef	_NOTRADLAYOUT
 	if (istradlayout()) {
 		Xlocate(TC_MARK, TL_PATH);
-		Xcprintf2("%<*d", TD_MARK, mark);
+		VOID_C XXcprintf("%<*d", TD_MARK, mark);
 		Xlocate(TC_MARK + TD_MARK + TW_MARK, TL_PATH);
-		Xcprintf2("%<'*qd", TD_SIZE, marksize);
+		VOID_C XXcprintf("%<'*qd", TD_SIZE, marksize);
 
 		return;
 	}
 #endif
 	Xlocate(C_MARK + W_MARK, L_STATUS);
-	Xcprintf2("%<*d", D_MARK, mark);
+	VOID_C XXcprintf("%<*d", D_MARK, mark);
 	if (sizeinfo) {
 		Xlocate(C_SIZE + W_SIZE, L_SIZE);
-		Xcprintf2("%<'*qd", D_SIZE, marksize);
+		VOID_C XXcprintf("%<'*qd", D_SIZE, marksize);
 	}
 }
 
@@ -674,7 +678,7 @@ CONST char *arg;
 {
 	int i;
 
-	if ((arg && atoi2(arg) == 0) || (!arg && mark)) {
+	if ((arg && Xatoi(arg) == 0) || (!arg && mark)) {
 		for (i = 0; i < maxfile; i++) filelist[i].tmpflags &= ~F_ISMRK;
 		mark = 0;
 		marksize = (off_t)0;
@@ -713,22 +717,22 @@ CONST char *arg;
 	return(FNC_UPDATE);
 }
 
-static reg_t *NEAR prepareregexp(mes, arg)
+static reg_ex_t *NEAR prepareregexp(mes, arg)
 CONST char *mes, *arg;
 {
-	reg_t *re;
+	reg_ex_t *re;
 	char *wild;
 
-	if (arg && *arg) wild = strdup2(arg);
+	if (arg && *arg) wild = Xstrdup(arg);
 	else if (!(wild = inputstr(mes, 0, 0, "*", -1))) return(NULL);
 	if (!*wild || strdelim(wild, 1)) {
 		warning(ENOENT, wild);
-		free2(wild);
+		Xfree(wild);
 		return(NULL);
 	}
 
 	re = regexp_init(wild, -1);
-	free2(wild);
+	Xfree(wild);
 
 	return(re);
 }
@@ -736,7 +740,7 @@ CONST char *mes, *arg;
 static int mark_find(arg)
 CONST char *arg;
 {
-	reg_t *re;
+	reg_ex_t *re;
 	int i;
 
 	if (!(re = prepareregexp(FINDF_K, arg))) return(FNC_CANCEL);
@@ -757,28 +761,29 @@ CONST char *arg;
 static int in_dir(arg)
 CONST char *arg;
 {
+	if (!isdir(&(filelist[filepos]))) return(warning_bell(arg));
 #ifndef	_NOARCHIVE
-	if (archivefile) /*EMPTY*/;
-	else
+	else if (archivefile) /*EMPTY*/;
 #endif
-	if (!isdir(&(filelist[filepos]))
-	|| isdotdir(filelist[filepos].name) == 2)
+	else if (isdotdir(filelist[filepos].name) == 2)
 		return(warning_bell(arg));
 
-	return(FNC_CHDIR);
+	return(dochdir4(filelist[filepos].name, 1));
 }
 
 /*ARGSUSED*/
 static int out_dir(arg)
 CONST char *arg;
 {
+	CONST char *path;
+
 #ifndef	_NOARCHIVE
-	if (archivefile) filepos = -1;
+	if (archivefile) path = NULL;
 	else
 #endif
-	replacefname(NULL);
+	path = parentpath;
 
-	return(FNC_CHDIR);
+	return(dochdir4(path, 1));
 }
 
 /*ARGSUSED*/
@@ -857,70 +862,28 @@ CONST char *arg;
 static int log_dir(arg)
 CONST char *arg;
 {
-#ifndef	_NOARCHIVE
-	CONST char *cp;
-	char dupfullpath[MAXPATHLEN];
-#endif
 	char *path;
+	int no;
 
-	if (arg && *arg) path = strdup2(arg);
+	if (arg && *arg) path = Xstrdup(arg);
 	else if (!(path = inputstr(LOGD_K, 0, -1, NULL, HST_PATH)))
 		return(FNC_CANCEL);
 	else if (!*(path = evalpath(path, 0))) {
-		free2(path);
+		Xfree(path);
 		return(FNC_CANCEL);
 	}
-#ifndef	_NOARCHIVE
-	if (archivefile && *path != '/') {
-		if (!(cp = archchdir(path))) {
-			warning(-1, path);
-			free2(path);
-			return(FNC_CANCEL);
-		}
-		free2(path);
-		if (cp != (char *)-1) filelist[filepos].name = (char *)cp;
-		else escapearch();
-		return(FNC_EFFECT);
-	}
-	else
-#endif
-	if (chdir3(path, 0) < 0) {
-		warning(-1, path);
-		free2(path);
-		return(FNC_CANCEL);
-	}
-	free2(path);
-#ifndef	_NOARCHIVE
-	if (archivefile) {
-		strcpy2(dupfullpath, fullpath);
-		while (archivefile) {
-# ifdef	_NOBROWSE
-			escapearch();
-# else
-			do {
-				escapearch();
-			} while (browselist);
-# endif
-		}
-		strcpy2(fullpath, dupfullpath);
-	}
-#endif
-	replacefname(NULL);
 
-	return(FNC_EFFECT);
+	no = dochdir4(path, 0);
+	Xfree(path);
+
+	return(no);
 }
 
 /*ARGSUSED*/
 static int log_top(arg)
 CONST char *arg;
 {
-	char *path;
-
-	path = strdup2(rootpath);
-	if (chdir3(path, 1) < 0) error(path);
-	replacefname(path);
-
-	return(FNC_EFFECT);
+	return(dochdir4(rootpath, 1));
 }
 
 static VOID NEAR clearscreen(VOID_A)
@@ -969,16 +932,16 @@ CONST char *file;
 	i = strlen3(file);
 	len = strlen2(buf);
 	if (i + len > n_lastcolumn) i = n_lastcolumn - len;
-	prompt = malloc2(i * KANAWID + strlen(buf) + 1);
-	strncpy3(prompt, file, &i, 0);
+	prompt = Xmalloc(i * KANAWID + strlen(buf) + 1);
+	VOID_C strncpy2(prompt, file, &i, 0);
 	strcat(prompt, buf);
 
 	i = min;
 	while ((buf = Xfgets(fp))) {
 		Xlocate(0, i);
 		Xputterm(L_CLEAR);
-		Xcprintf2("%.*k", n_column, buf);
-		free2(buf);
+		VOID_C XXcprintf("%^.*k", n_column, buf);
+		Xfree(buf);
 		if (++i >= max) {
 			i = min;
 			hideclock = 1;
@@ -992,8 +955,8 @@ CONST char *file;
 			Xputterm(L_CLEAR);
 		}
 	}
-	Xfclose(fp);
-	free2(prompt);
+	VOID_C Xfclose(fp);
+	Xfree(prompt);
 }
 #endif	/* !PAGER */
 
@@ -1047,7 +1010,7 @@ static int NEAR execshell(VOID_A)
 	sigvecset(0);
 	mode = Xtermmode(0);
 	wastty = isttyiomode;
-	kanjiputs(SHEXT_K);
+	VOID_C Xkanjiputs(SHEXT_K);
 #ifdef	DEP_PTY
 	if (isptymode()) {
 		min = filetop(win);
@@ -1067,7 +1030,7 @@ static int NEAR execshell(VOID_A)
 	checkscreen(-1, -1);
 #endif
 	if (wastty) Xttyiomode(wastty - 1);
-	Xtermmode(mode);
+	VOID_C Xtermmode(mode);
 	sigvecset(1);
 
 	return(ret);
@@ -1163,8 +1126,8 @@ CONST char *arg;
 static int sort_dir(arg)
 CONST char *arg;
 {
-	CONST char *str[6];
-	int i, tmp1, tmp2, val[6], *dupl;
+	CONST char *str[MAXSORTTYPE + 1];
+	int i, n, tmp1, tmp2, val[MAXSORTTYPE + 1], *dupl;
 
 	str[0] = ONAME_K;
 	str[1] = OEXT_K;
@@ -1172,29 +1135,30 @@ CONST char *arg;
 	str[3] = ODATE_K;
 	str[4] = OLEN_K;
 	str[5] = ORAW_K;
-	val[0] = 1;
-	val[1] = 2;
-	val[2] = 3;
-	val[3] = 4;
-	val[4] = 5;
+	val[0] = SRT_FILENAME;
+	val[1] = SRT_EXTENSION;
+	val[2] = SRT_SIZE;
+	val[3] = SRT_DATE;
+	val[4] = SRT_LENGTH;
 	val[5] = 0;
 
-	tmp1 = sorton & 7;
-	tmp2 = sorton & ~7;
+	tmp1 = (sorton & SRT_TYPE);
+	tmp2 = (sorton & SRT_DESC);
 	if (arg && *arg) {
-		i = atoi2(arg);
-		if ((i >= 0 && i <= 13) || (i & 7) <= 5) {
-			tmp1 = i & 7;
-			tmp2 = i & ~7;
+		n = Xatoi(arg);
+		if ((n >= 0 && n <= (MAXSORTTYPE | SRT_DESC))
+		|| (n & SRT_TYPE) <= MAXSORTTYPE) {
+			tmp1 = (n & SRT_TYPE);
+			tmp2 = (n & SRT_DESC);
 		}
 	}
 	else {
-		if (tmp1) i = 6;
-		else {
-			i = 5;
+		n = arraysize(str);
+		if (!tmp1) {
+			n--;
 			tmp1 = val[0];
 		}
-		if (selectstr(&tmp1, i, 0, str, val) != K_CR)
+		if (selectstr(&tmp1, n, 0, str, val) != K_CR)
 			return(FNC_CANCEL);
 	}
 
@@ -1206,11 +1170,11 @@ CONST char *arg;
 		str[0] = OINC_K;
 		str[1] = ODEC_K;
 		val[0] = 0;
-		val[1] = 8;
+		val[1] = SRT_DESC;
 		if (selectstr(&tmp2, 2, 56, str, val) != K_CR)
 			return(FNC_CANCEL);
-		sorton = tmp1 + tmp2;
-		dupl = (int *)malloc2(maxfile * sizeof(int));
+		sorton = (tmp1 | tmp2);
+		dupl = (int *)Xmalloc(maxfile * sizeof(int));
 		for (i = 0; i < maxfile; i++) {
 			dupl[i] = filelist[i].ent;
 			filelist[i].ent = i;
@@ -1221,7 +1185,7 @@ CONST char *arg;
 			tmp1 = filelist[i].ent;
 			filelist[i].ent = dupl[tmp1];
 		}
-		free2(dupl);
+		Xfree(dupl);
 		chgorder = 1;
 	}
 
@@ -1323,7 +1287,7 @@ CONST char *arg;
 	if (!fdmode) {
 		Xlocate(0, L_MESLINE);
 		Xputterm(L_CLEAR);
-		Xkanjiputs(QUIT_K);
+		VOID_C Xkanjiputs(QUIT_K);
 		val[0] = 0;
 		val[1] = 1;
 		val[2] = 2;
@@ -1349,16 +1313,16 @@ CONST char *arg;
 {
 	char *path;
 
-	if (arg && *arg) path = strdup2(arg);
+	if (arg && *arg) path = Xstrdup(arg);
 	else if (!(path = inputstr(MAKED_K, 1, -1, NULL, HST_PATH)))
 		return(FNC_CANCEL);
 	else if (!*(path = evalpath(path, 0))) {
-		free2(path);
+		Xfree(path);
 		return(FNC_CANCEL);
 	}
 
 	if (mkdir2(path, 0777) < 0) warning(-1, path);
-	free2(path);
+	Xfree(path);
 
 	return(FNC_EFFECT);
 }
@@ -1403,11 +1367,11 @@ CONST char *arg;
 
 	if (isdotdir(filelist[filepos].name)) return(warning_bell(arg));
 	if (arg && *arg) {
-		file = strdup2(arg);
+		file = Xstrdup(arg);
 		errno = EEXIST;
 		if (Xaccess(file, F_OK) >= 0 || errno != ENOENT) {
 			warning(-1, file);
-			free2(file);
+			Xfree(file);
 			return(FNC_CANCEL);
 		}
 	}
@@ -1419,19 +1383,19 @@ CONST char *arg;
 			warning(-1, file);
 		}
 		else warning(0, WRONG_K);
-		free2(file);
+		Xfree(file);
 	}
 
 	if (Xrename(fnodospath(path, filepos), file) < 0) {
 		warning(-1, file);
-		free2(file);
+		Xfree(file);
 		return(FNC_CANCEL);
 	}
 	if (strdelim(file, 0)) {
-		free2(file);
-		file = strdup2(parentpath);
+		Xfree(file);
+		file = Xstrdup(parentpath);
 	}
-	replacefname(file);
+	setlastfile(file);
 
 	return(FNC_EFFECT);
 }
@@ -1500,7 +1464,7 @@ CONST char *arg;
 {
 	char *wild;
 
-	if (arg && *arg) wild = strdup2(arg);
+	if (arg) wild = Xstrdup(arg);
 	else if (!(wild = inputstr(FINDF_K, 0, 0, "*", -1)))
 		return(FNC_CANCEL);
 
@@ -1510,13 +1474,13 @@ CONST char *arg;
 #endif
 	if (strdelim(wild, 1)) {
 		warning(ENOENT, wild);
-		free2(wild);
+		Xfree(wild);
 		return(FNC_CANCEL);
 	}
-	free2(findpattern);
+	Xfree(findpattern);
 	if (*wild) findpattern = wild;
 	else {
-		free2(wild);
+		Xfree(wild);
 		findpattern = NULL;
 	}
 
@@ -1540,11 +1504,14 @@ CONST char *arg;
 	if (!(tmp = strrdelim(destpath, 0))) tmp = destpath;
 	else {
 		*(tmp++) = '\0';
-		chdir3(destpath, 1);
+		if (dochdir4(destpath, 1) == FNC_CANCEL) {
+			Xfree(destpath);
+			return(FNC_CANCEL);
+		}
 	}
 
-	replacefname(strdup2(tmp));
-	free2(destpath);
+	setlastfile(tmp);
+	Xfree(destpath);
 
 	return(FNC_EFFECT);
 }
@@ -1565,7 +1532,7 @@ CONST char *arg;
 	char *com;
 	int ret;
 
-	if (arg) com = strdup2(arg);
+	if (arg) com = Xstrdup(arg);
 	else {
 		Xttyiomode(1);
 		com = inputshellloop(-1, NULL);
@@ -1573,14 +1540,16 @@ CONST char *arg;
 		if (!com) return(FNC_CANCEL);
 	}
 	if (*com) {
-		ret = ptyusercomm(com, filelist[filepos].name, F_ARGSET);
+		ret = ptyusercomm(com,
+			(filepos < maxfile) ? filelist[filepos].name : NULL,
+			F_ARGSET);
 		ret = evalstatus(ret);
 	}
 	else {
 		execshell();
 		ret = FNC_EFFECT;
 	}
-	free2(com);
+	Xfree(com);
 
 	return(ret);
 }
@@ -1624,20 +1593,20 @@ CONST char *arg;
 	}
 	else {
 		len = strlen(filelist[filepos].name) + 2 + 1;
-		tmp = malloc2(len + 1);
+		tmp = Xmalloc(len + 1);
 		tmp[0] = '.';
 		tmp[1] = _SC_;
-		strcpy2(&(tmp[2]), filelist[filepos].name);
+		Xstrcpy(&(tmp[2]), filelist[filepos].name);
 	}
 	Xttyiomode(1);
 	com = inputshellloop(len, tmp);
 	Xttyiomode(0);
-	if (tmp != filelist[filepos].name) free2(tmp);
+	if (tmp != filelist[filepos].name) Xfree(tmp);
 #endif
 	if (!com) return(FNC_CANCEL);
 	if (!*com) {
 		execshell();
-		free2(com);
+		Xfree(com);
 		return(FNC_EFFECT);
 	}
 
@@ -1665,7 +1634,7 @@ CONST char *arg;
 		ret = ptyusercomm(com, filelist[filepos].name, F_ARGSET);
 	}
 	ret = evalstatus(ret);
-	free2(com);
+	Xfree(com);
 
 	return(ret);
 }
@@ -1685,16 +1654,16 @@ CONST char *arg;
 	char *file;
 	int ret;
 
-	if (arg && *arg) file = strdup2(arg);
+	if (arg && *arg) file = Xstrdup(arg);
 	else if (!(file = inputstr(PACK_K, 1, -1, NULL, HST_PATH)))
 		return(FNC_CANCEL);
 	else if (!*(file = evalpath(file, 0))) {
-		free2(file);
+		Xfree(file);
 		return(FNC_CANCEL);
 	}
 
 	ret = pack(file);
-	free2(file);
+	Xfree(file);
 	if (ret < 0) {
 		Xputterm(T_BELL);
 		return(FNC_CANCEL);
@@ -1753,16 +1722,16 @@ CONST char *arg;
 	char *path;
 	int ret;
 
-	if (arg && *arg) path = strdup2(arg);
+	if (arg && *arg) path = Xstrdup(arg);
 	else if (!(path = inputstr(FSDIR_K, 0, -1, NULL, HST_PATH)))
 		return(FNC_CANCEL);
 	else if (!*(path = evalpath(path, 0))) {
-		free2(path);
-		path = strdup2(curpath);
+		Xfree(path);
+		path = Xstrdup(curpath);
 	}
 
 	ret = infofs(path);
-	free2(path);
+	Xfree(path);
 	if (!ret) return(FNC_CANCEL);
 
 	return(FNC_HELPSPOT);
@@ -1786,12 +1755,10 @@ CONST char *s;
 	n = ATR_MODEONLY;
 	Xlocate(0, ATTR_Y);
 	Xputterm(L_CLEAR);
-#if	!defined (_NOEXTRAATTR) && !defined (NOUID)
-	Xputterm(T_STANDOUT);
-#endif
-	Xkanjiputs(s);
-#if	!defined (_NOEXTRAATTR) && !defined (NOUID)
-	Xputterm(END_STANDOUT);
+#if	defined (_NOEXTRAATTR) || defined (NOUID)
+	VOID_C Xkanjiputs(s);
+#else
+	VOID_C Xattrkanjiputs(s, 1);
 #endif
 	if (selectstr(&n, MAXATTRSEL, ATTR_X, str, val) != K_CR) return(-1);
 
@@ -1862,19 +1829,14 @@ static int tree_dir(arg)
 CONST char *arg;
 {
 	char *path;
+	int no;
 
 	if (!(path = tree(0, NULL))) return(FNC_UPDATE);
-	if (chdir3(path, 1) < 0) {
-		warning(-1, path);
-		free2(path);
-		return(FNC_UPDATE);
-	}
-	free2(path);
-	free2(findpattern);
-	findpattern = NULL;
-	replacefname(NULL);
+	no = dochdir4(path, 1);
+	if (no == FNC_CANCEL) no = FNC_UPDATE;
+	Xfree(path);
 
-	return(FNC_EFFECT);
+	return(no);
 }
 #endif	/* !_NOTREE */
 
@@ -1896,7 +1858,7 @@ CONST char *arg;
 #ifdef	DEP_PSEUDOPATH
 	drive = 0;
 #endif
-	if (arg && *arg) dev = strdup2(arg);
+	if (arg && *arg) dev = Xstrdup(arg);
 #if	MSDOS
 	else if (!(dev = inputstr(BKUP_K, 1, -1, NULL, HST_PATH)))
 		return(FNC_CANCEL);
@@ -1905,18 +1867,18 @@ CONST char *arg;
 		return(FNC_CANCEL);
 #endif
 	else if (!*(dev = evalpath(dev, 0))) {
-		free2(dev);
+		Xfree(dev);
 		return(FNC_CANCEL);
 	}
 	else if (archivefile) {
 		if (!(dir = tmpunpack(0))) {
-			free2(dev);
+			Xfree(dev);
 			return(FNC_CANCEL);
 		}
 	}
 #ifdef	DEP_PSEUDOPATH
 	else if ((drive = tmpdosdupl(nullstr, &dir, 0)) < 0) {
-		free2(dev);
+		Xfree(dev);
 		return(FNC_CANCEL);
 	}
 #endif
@@ -1928,7 +1890,7 @@ CONST char *arg;
 #endif
 	if (archivefile) removetmp(dir, NULL);
 
-	free2(dev);
+	Xfree(dev);
 	if (ret <= 0) return(FNC_CANCEL);
 
 	return(FNC_EFFECT);
@@ -1942,10 +1904,8 @@ CONST char *arg;
 	isearch = 1;
 	Xlocate(0, L_HELP);
 	Xputterm(L_CLEAR);
-	Xputterm(T_STANDOUT);
-	win_x = Xkanjiputs(SEAF_K);
+	win_x = Xattrkanjiputs(SEAF_K, 1);
 	win_y = L_HELP;
-	Xputterm(END_STANDOUT);
 
 	return(FNC_NONE);
 }
@@ -1957,10 +1917,8 @@ CONST char *arg;
 	isearch = -1;
 	Xlocate(0, L_HELP);
 	Xputterm(L_CLEAR);
-	Xputterm(T_STANDOUT);
-	win_x = Xkanjiputs(SEAB_K);
+	win_x = Xattrkanjiputs(SEAB_K, 1);
 	win_y = L_HELP;
-	Xputterm(END_STANDOUT);
 
 	return(FNC_NONE);
 }
@@ -1980,7 +1938,8 @@ int oldwin;
 	filelist = addlist(filelist, maxfile, &maxent);
 	memcpy((char *)filelist, (char *)(winvar[oldwin].v_filelist), i);
 	for (i = 0; i < winvar[oldwin].v_maxfile; i++)
-		filelist[i].name = strdup2(winvar[oldwin].v_filelist[i].name);
+		filelist[i].name = Xstrdup(winvar[oldwin].v_filelist[i].name);
+	lastfile = Xstrdup(winvar[oldwin].v_lastfile);
 	filepos = winvar[oldwin].v_filepos;
 	sorton = winvar[oldwin].v_sorton;
 	dispmode = winvar[oldwin].v_dispmode;
@@ -1989,14 +1948,14 @@ int oldwin;
 static VOID NEAR movewin(oldwin)
 int oldwin;
 {
-	free2(winvar[oldwin].v_fullpath);
-	winvar[oldwin].v_fullpath = strdup2(fullpath);
-	if (winvar[win].v_fullpath) strcpy2(fullpath, winvar[win].v_fullpath);
+	Xfree(winvar[oldwin].v_fullpath);
+	winvar[oldwin].v_fullpath = Xstrdup(fullpath);
+	if (winvar[win].v_fullpath) Xstrcpy(fullpath, winvar[win].v_fullpath);
 # ifndef	_NOARCHIVE
-	free2(winvar[oldwin].v_archivedir);
-	winvar[oldwin].v_archivedir = strdup2(archivedir);
+	Xfree(winvar[oldwin].v_archivedir);
+	winvar[oldwin].v_archivedir = Xstrdup(archivedir);
 	if (winvar[win].v_archivedir)
-		strcpy2(archivedir, winvar[win].v_archivedir);
+		Xstrcpy(archivedir, winvar[win].v_archivedir);
 # endif
 }
 
@@ -2005,6 +1964,8 @@ int nextwin(VOID_A)
 	int oldwin;
 
 	if (windows <= 1) return(-1);
+
+	if (lastfile || internal_status == FNC_EFFECT) getfilelist();
 	oldwin = win;
 	if (++win >= windows) win = 0;
 	duplwin(oldwin);
@@ -2046,7 +2007,7 @@ CONST char *arg;
 # ifdef	_NOEXTRAWIN
 	else if (fileperrow(windows + 1) < WFILEMIN)
 # else
-	else if (winvar[win].v_fileperrow < WFILEMIN * 2 + 1)
+	else if (FILEPERROW < WFILEMIN * 2 + 1)
 # endif
 	{
 		warning(0, NOROW_K);
@@ -2065,8 +2026,7 @@ CONST char *arg;
 		calcwin();
 # else
 		i = (winvar[oldwin].v_fileperrow - 1) / 2;
-		winvar[win].v_fileperrow =
-			(winvar[oldwin].v_fileperrow - 1) - i;
+		FILEPERROW = (winvar[oldwin].v_fileperrow - 1) - i;
 		winvar[oldwin].v_fileperrow = i;
 # endif
 
@@ -2105,7 +2065,7 @@ CONST char *arg;
 	int n, next;
 
 	if (windows <= 1) return(FNC_NONE);
-	if (!arg || (n = atoi2(arg)) <= 0) n = 1;
+	if (!arg || (n = Xatoi(arg)) <= 0) n = 1;
 	if ((next = win + 1) >= windows) next = 0;
 	if (winvar[next].v_fileperrow - n < WFILEMIN)
 		n = winvar[next].v_fileperrow - WFILEMIN;
@@ -2126,7 +2086,7 @@ CONST char *arg;
 	int n, next;
 
 	if (windows <= 1) return(FNC_NONE);
-	if (!arg || (n = atoi2(arg)) <= 0) n = 1;
+	if (!arg || (n = Xatoi(arg)) <= 0) n = 1;
 	if ((next = win + 1) >= windows) next = 0;
 	if (FILEPERROW - n < WFILEMIN) n = FILEPERROW - WFILEMIN;
 	if (n <= 0) return(warning_bell(arg));
