@@ -36,6 +36,7 @@
 #endif	/* !MSDOS */
 
 #define	CLOCKUPDATE		10	/* sec */
+#define	SWITCHUSER		"su"
 #ifndef	BINDIR
 #define	BINDIR			"/usr/local/bin"
 #endif
@@ -59,6 +60,11 @@ extern u_int _stklen = 0x5800;
 #define	harderr_t		int
 #endif
 
+typedef struct _sigfunc_t {
+	int sig;
+	int (*func)__P_((VOID_A));
+} sigfunc_t;
+
 #if	MSDOS
 # ifndef	BSPATHDELIM
 extern char *adjustpname __P_((char *));
@@ -213,9 +219,6 @@ char *tmpfilename = NULL;
 int adjtty = 0;
 #endif
 int showsecond = 0;
-#if	FD >= 2
-int thruargs = 0;
-#endif
 int hideclock = 0;
 #ifdef	SIGALRM
 int noalrm = 1;
@@ -233,6 +236,57 @@ static int timersec = 0;
 #ifdef	SIGWINCH
 static int nowinch = 0;
 #endif
+static CONST sigfunc_t sigfunclist[] = {
+#ifdef	SIGINT
+	{SIGINT, ignore_int},
+#endif
+#ifdef	SIGQUIT
+	{SIGQUIT, ignore_quit},
+#endif
+#ifdef	SIGCONT
+	{SIGCONT, ignore_cont},
+#endif
+#ifdef	SIGHUP
+	{SIGHUP, hanguperror},
+#endif
+#ifdef	SIGILL
+	{SIGILL, illerror},
+#endif
+#ifdef	SIGTRAP
+	{SIGTRAP, traperror},
+#endif
+#ifdef	SIGIOT
+	{SIGIOT, ioerror},
+#endif
+#ifdef	SIGEMT
+	{SIGEMT, emuerror},
+#endif
+#ifdef	SIGFPE
+	{SIGFPE, floaterror},
+#endif
+#ifdef	SIGBUS
+	{SIGBUS, buserror},
+#endif
+#ifdef	SIGSEGV
+	{SIGSEGV, segerror},
+#endif
+#ifdef	SIGSYS
+	{SIGSYS, syserror},
+#endif
+#ifdef	SIGPIPE
+	{SIGPIPE, pipeerror},
+#endif
+#ifdef	SIGTERM
+	{SIGTERM, terminate},
+#endif
+#ifdef	SIGXCPU
+	{SIGXCPU, xcpuerror},
+#endif
+#ifdef	SIGXFSZ
+	{SIGXFSZ, xsizerror},
+#endif
+	{-1, NULL}
+};
 
 
 #if	MSDOS && !defined (PROTECTED_MODE)
@@ -261,10 +315,10 @@ CONST char *s;
 	if (isorgpid()) {
 		forcecleandir(deftmpdir, tmpfilename);
 #ifdef	DEP_DOSDRIVE
-		dosallclose();
+		VOID_C dosallclose();
 #endif
 #ifdef	DEP_URLPATH
-		urlallclose();
+		VOID_C urlallclose();
 #endif
 		Xstdiomode();
 		endterm();
@@ -276,9 +330,9 @@ CONST char *s;
 	if (errno) perror2(s);
 	else {
 		Xfputs(s, Xstderr);
-		fputnl(Xstderr);
+		VOID_C fputnl(Xstderr);
 	}
-	Xfclose(Xstderr);
+	VOID_C Xfclose(Xstderr);
 	doing = 2;
 
 	if (isorgpid()) {
@@ -321,12 +375,13 @@ int sig;
 	signal2(sig, SIG_IGN);
 
 	if (isorgpid()) {
+		printf_urgent++;
 		forcecleandir(deftmpdir, tmpfilename);
 #ifdef	DEP_DOSDRIVE
-		dosallclose();
+		VOID_C dosallclose();
 #endif
 #ifdef	DEP_URLPATH
-		urlallclose();
+		VOID_C urlallclose();
 #endif
 #ifdef	DEP_LOGGING
 		endlog(sig + 128);
@@ -350,6 +405,7 @@ int sig;
 		closetty(&ttyio, &ttyout);
 		muntrace();
 #endif	/* DEBUG */
+		printf_urgent--;
 	}
 
 	signal2(sig, SIG_DFL);
@@ -549,11 +605,11 @@ VOID setlinecol(VOID_A)
 	char buf[MAXLONGWIDTH + 1];
 
 	if (getconstvar(ENVLINES)) {
-		snprintf2(buf, sizeof(buf), "%d", n_line);
+		VOID_C Xsnprintf(buf, sizeof(buf), "%d", n_line);
 		setenv2(ENVLINES, buf, 1);
 	}
 	if (getconstvar(ENVCOLUMNS)) {
-		snprintf2(buf, sizeof(buf), "%d", n_column);
+		VOID_C Xsnprintf(buf, sizeof(buf), "%d", n_column);
 		setenv2(ENVCOLUMNS, buf, 1);
 	}
 }
@@ -630,7 +686,7 @@ int xmax, ymax;
 	for (i = 0;; i++) {
 		if (!(cp = getwsize(xmax, ymax))) {
 			if (n_line >= row) break;
-			cp = NOROW_K;
+			cp = "Line size is too small for operation.";
 		}
 		if (!i) {
 			Xputterms(T_CLEAR);
@@ -638,24 +694,21 @@ int xmax, ymax;
 			keyflush();
 		}
 
-		if (i & 1) Xcputs2(SCRSZ_K);
-		else {
-			Xputterm(T_STANDOUT);
-			Xcputs2(cp);
-			Xputterm(END_STANDOUT);
-		}
+		if (i & 1)
+			cp = "Terminate with the ESC key, or widen a screen.";
+		Xattrputs(cp, !(i & 1));
 		Xputterm(T_BELL);
 		Xcputnl();
 		Xtflush();
 		if (kbhit2(1000000L) && getkey3(0, inputkcode, 0) == K_ESC) {
 			errno = 0;
-			error(INTR_K);
+			error("Interrupted by the ESC key.");
 		}
 	}
 	dumbterm = dupdumbterm;
 	loadtermio(ttyio, tty, NULL);
 	isttyiomode = wastty;
-	free2(tty);
+	Xfree(tty);
 
 	setlinecol();
 	if (n_line != dupn_line) {
@@ -692,7 +745,7 @@ int forced;
 		checkscreen(-1, -1);
 		if (isorgpid()) {
 			if (x != n_column || y != n_line) rewritefile(1);
-			if (subwindow) ungetkey2(K_CTRL('L'));
+			if (subwindow) VOID_C ungetkey2(K_CTRL('L'), 1);
 		}
 	}
 }
@@ -720,7 +773,7 @@ int hide;
 
 	if (timersec) now++;
 	else {
-		now = time2();
+		now = Xtime(NULL);
 		timersec = CLOCKUPDATE;
 	}
 	if (timersec-- < CLOCKUPDATE && !showsecond) return;
@@ -745,11 +798,11 @@ int hide;
 	if (!isleftshift()) x--;
 	Xlocate(x, L_TITLE);
 	Xputterm(T_STANDOUT);
-	Xcprintf2("%02d-%02d-%02d %02d:%02d",
+	VOID_C XXcprintf("%02d-%02d-%02d %02d:%02d",
 		tm -> tm_year % 100,
 		tm -> tm_mon + 1, tm -> tm_mday,
 		tm -> tm_hour, tm -> tm_min);
-	if (showsecond) Xcprintf2(":%02d", tm -> tm_sec);
+	if (showsecond) VOID_C XXcprintf(":%02d", tm -> tm_sec);
 	Xputterm(END_STANDOUT);
 	Xlocate(win_x, win_y);
 	Xtflush();
@@ -850,10 +903,10 @@ int *lenp;
 {
 	char *cp, *eol;
 
-	cp = strchr2(version, ' ');
+	cp = Xstrchr(version, ' ');
 	while (*(++cp) == ' ');
 	if (lenp) {
-		if (!(eol = strchr2(cp, ' '))) eol = cp + strlen(cp);
+		if (!(eol = Xstrchr(cp, ' '))) eol = cp + strlen(cp);
 		*lenp = eol - cp;
 	}
 
@@ -869,27 +922,27 @@ VOID title(VOID_A)
 	Xputterm(T_STANDOUT);
 	len = 0;
 	if (!isleftshift()) {
-		Xputch2(' ');
+		VOID_C XXputch(' ');
 		len++;
 	}
-	Xcputs2(" FD");
+	XXcputs(" FD");
 	len += 3;
 	if (!ishardomit()) {
-		Xcputs2("(File & Directory tool)");
+		XXcputs("(File & Directory tool)");
 		len += 23;
 	}
-	Xcputs2(" Ver.");
+	XXcputs(" Ver.");
 	len += 5;
 	cp = getversion(&n);
 	cputstr(n, cp);
 	if (distributor) {
-		Xputch2('#');
+		VOID_C XXputch('#');
 		n++;
 	}
-	cp = (iswellomit()) ? nullstr : " (c)1995-2008 T.Shirai  ";
-	Xcputs2(cp);
+	cp = (iswellomit()) ? nullstr : " (c)1995-2013 T.Shirai  ";
+	XXcputs(cp);
 	n = n_column - len - strlen2(cp) - n;
-	while (n-- > 0) Xputch2(' ');
+	while (n-- > 0) VOID_C XXputch(' ');
 	Xputterm(END_STANDOUT);
 	timersec = 0;
 	printtime(0);
@@ -922,9 +975,9 @@ VOID saveorigenviron(VOID_A)
 
 # if	FD >= 3
 	cp = getversion(&n);
-	cp = strndup2(cp, n);
+	cp = Xstrndup(cp, n);
 	setenv2(FDVERSION, cp, 0);
-	free2(cp);
+	Xfree(cp);
 # endif
 }
 #endif	/* !_NOCUSTOMIZE */
@@ -944,10 +997,13 @@ char *CONST *argv;
 	for (i = 1; argv && argv[i]; i++) {
 		if (cp > buf) *(cp++) = ' ';
 		tmp = killmeta(argv[i]);
-		len = snprintf2(cp, (int)sizeof(buf) - (cp - buf), "%s", tmp);
-		free2(tmp);
+		len = Xsnprintf(cp, (int)sizeof(buf) - (cp - buf), "%s", tmp);
+		Xfree(tmp);
 		if (len < 0) break;
-		cp += strlen(cp);
+#ifdef	CODEEUC
+		len = strlen(cp);
+#endif
+		cp += len;
 	}
 	*cp = '\0';
 
@@ -958,7 +1014,7 @@ char *CONST *argv;
 # else
 	lvl = ((uid = getuid())) ? _LOG_DEBUG_ : _LOG_WARNING_;
 	logmessage(lvl, "%s (%-*.*s) starts; UID=%d; PWD=%k; ARGS=%k",
-		progname, len, len, cp, uid, origpath, buf);
+		progname, len, len, cp, (int)uid, origpath, buf);
 # endif
 }
 
@@ -971,14 +1027,14 @@ int status;
 # endif
 	char cwd[MAXPATHLEN];
 
-	if (!Xgetwd(cwd)) strcpy2(cwd, "?");
+	if (!Xgetwd(cwd)) Xstrcpy(cwd, "?");
 # ifdef	NOUID
 	logmessage(_LOG_DEBUG_, "%s ends; PWD=%k; STATUS=%d",
 		progname, cwd, status);
 # else
 	lvl = ((uid = getuid())) ? _LOG_DEBUG_ : _LOG_WARNING_;
 	logmessage(lvl, "%s ends; UID=%d; PWD=%k; STATUS=%d",
-		progname, uid, cwd, status);
+		progname, (int)uid, cwd, status);
 # endif
 	logclose();
 }
@@ -998,13 +1054,13 @@ CONST char *line;
 	else i = execpseudoshell(cp, F_IGNORELIST | F_NOCOMLINE);
 	if (i) {
 		Xputterm(L_CLEAR);
-		Xcprintf2("%s, line %d: %s", file, n, ILFNC_K);
+		VOID_C XXcprintf("%s, line %d: %k", file, n, ILFNC_K);
 		Xcputnl();
 		Xputterm(L_CLEAR);
-		Xcprintf2("\t%s", line);
+		VOID_C XXcprintf("\t%s", line);
 		Xcputnl();
 	}
-	free2(command);
+	Xfree(command);
 
 	return((i) ? -1 : 0);
 }
@@ -1033,26 +1089,26 @@ int exist;
 # if	!MSDOS
 	tmp = NULL;
 	if (!exist && (tmp = getconstvar(ENVTERM))) {
-		cp = malloc2(strlen(file) + strlen(tmp) + 1 + 1);
-		strcpy2(strcpy2(strcpy2(cp, file), "."), tmp);
+		cp = Xmalloc(strlen(file) + strlen(tmp) + 1 + 1);
+		Xstrcpy(Xstrcpy(Xstrcpy(cp, file), "."), tmp);
 		tmp = evalpath(cp, 0);
 		if (stat2(tmp, &st) < 0 || !s_isreg(&st)) {
-			free2(tmp);
+			Xfree(tmp);
 			tmp = NULL;
 		}
 	}
 	if (!tmp)
 # endif	/* !MSDOS */
-	tmp = evalpath(strdup2(file), 0);
+	tmp = evalpath(Xstrdup(file), 0);
 	fp = Xfopen(tmp, "r");
 	if (!fp) {
 		if (!exist) {
-			free2(tmp);
+			Xfree(tmp);
 			return(0);
 		}
-		Xcprintf2("%s: Not found", tmp);
+		VOID_C XXcprintf("%s: Not found", tmp);
 		Xcputnl();
-		free2(tmp);
+		Xfree(tmp);
 		return(-1);
 	}
 
@@ -1061,12 +1117,12 @@ int exist;
 	while ((line = Xfgets(fp))) {
 		n++;
 		if (*line == ';' || *line == '#') {
-			free2(line);
+			Xfree(line);
 			continue;
 		}
 
 		cp = line + strlen(line);
-		for (cp--; cp >= line; cp--) if (!isblank2(*cp)) break;
+		for (cp--; cp >= line; cp--) if (!Xisblank(*cp)) break;
 		cp[1] = '\0';
 
 		cont = 0;
@@ -1079,23 +1135,23 @@ int exist;
 
 		if (!fold) fold = line;
 		else if (*line) {
-			fold = realloc2(fold, strlen(fold) + strlen(line) + 1);
+			fold = Xrealloc(fold, strlen(fold) + strlen(line) + 1);
 			strcat(fold, line);
 		}
 
 		if (cont) {
-			if (fold != line) free2(line);
+			if (fold != line) Xfree(line);
 			continue;
 		}
 
 		if (execruncomline(fold, tmp, n, line) < 0) er++;
-		if (fold != line) free2(line);
+		if (fold != line) Xfree(line);
 		fold = NULL;
 	}
 
 	if (fold && execruncomline(fold, tmp, n, line) < 0) er++;
-	Xfclose(fp);
-	free2(tmp);
+	VOID_C Xfclose(fp);
+	Xfree(tmp);
 #endif	/* !DEP_ORIGSHELL */
 
 	return(er ? -1 : 0);
@@ -1109,7 +1165,7 @@ char *CONST argv[];
 	int i, optc;
 
 	optc = 1;
-	optv = (char **)malloc2((argc + 1) * sizeof(char *));
+	optv = (char **)Xmalloc((argc + 1) * sizeof(char *));
 	optv[0] = argv[0];
 
 	for (i = 1; i < argc; i++) {
@@ -1134,7 +1190,7 @@ char *CONST argv[];
 #ifdef	DEP_ORIGSHELL
 	if (initshell(optc, optv) < 0) Xexit2(RET_FAIL);
 #endif
-	free2(optv);
+	Xfree(optv);
 
 	return(argc);
 }
@@ -1151,10 +1207,10 @@ char *CONST argv[];
 			i++;
 			break;
 		}
-		tmp = strdup2(&(argv[i][1]));
-		if ((cp = strchr2(tmp, '='))) *(cp++) = '\0';
+		tmp = Xstrdup(&(argv[i][1]));
+		if ((cp = Xstrchr(tmp, '='))) *(cp++) = '\0';
 		setenv2(tmp, cp, 0);
-		free2(tmp);
+		Xfree(tmp);
 	}
 
 	return(i);
@@ -1179,26 +1235,30 @@ static char *NEAR searchexecname(argv, envp)
 CONST char *argv;
 char *CONST envp[];
 {
-	CONST char *env;
+	CONST char *s, *env;
 	char *cp;
 	int n;
 
 	n = 0;
 	if (!(env = searchenv(ENVPATH, envp))) env = DEFPATH;
-	if ((cp = searchexecpath(argv, env))) return(cp);
-	if (*argv == '-') {
-		if ((cp = searchexecpath(++argv, env))) return(cp);
-		n++;
-	}
-	if (*argv == 'r' && (cp = searchexecpath(++argv, env))) return(cp);
-	if ((cp = searchexecpath(FDSHELL, env))) return(cp);
-	if ((cp = searchexecpath(FDPROG, env))) return(cp);
-	if (n) {
-		getlogininfo(NULL, &env);
-		cp = strdup2(env);
+	s = progname;
+	if (*s == '-') s++;
+	if (strpathcmp(s, SWITCHUSER)) {
+		if ((cp = searchexecpath(argv, env))) return(cp);
+		if (*argv == '-') {
+			if ((cp = searchexecpath(++argv, env))) return(cp);
+			n++;
+		}
+		if (*argv == 'r' && (cp = searchexecpath(++argv, env)))
+			return(cp);
+		if ((cp = searchexecpath(FDSHELL, env))) return(cp);
+		if ((cp = searchexecpath(FDPROG, env))) return(cp);
+
+		if (!n) return(NULL);
 	}
 
-	return(cp);
+	getlogininfo(NULL, &s);
+	return(Xstrdup(s));
 }
 #endif	/* !MSDOS */
 
@@ -1211,11 +1271,11 @@ CONST char *argv;
 
 	progname = getbasename(argv);
 #if	MSDOS || defined (CYGWIN)
-	if ((cp = strchr2(progname, '.')) && cp > progname)
-		progname = strndup2(progname, cp - progname);
+	if ((cp = Xstrchr(progname, '.')) && cp > progname)
+		progname = Xstrndup(progname, cp - progname);
 	else
 #endif
-	progname = strdup2(progname);
+	progname = Xstrdup(progname);
 }
 
 static VOID NEAR setexecpath(argv, envp)
@@ -1225,18 +1285,21 @@ char *CONST envp[];
 	CONST char *cp;
 	char *tmp, buf[MAXPATHLEN];
 
-	if (!Xgetwd(buf)) error(NOCWD_K);
-	origpath = strdup2(buf);
+	if (!Xgetwd(buf)) {
+		errno = 0;
+		error("No current working directory.");
+	}
+	origpath = Xstrdup(buf);
 	if ((cp = searchenv(ENVPWD, envp))) {
 		*fullpath = '\0';
-		realpath2(cp, fullpath, 0);
-		realpath2(fullpath, buf, RLP_READLINK);
+		Xrealpath(cp, fullpath, 0);
+		Xrealpath(fullpath, buf, RLP_READLINK);
 		if (!strpathcmp(origpath, buf)) {
-			free2(origpath);
-			origpath = strdup2(fullpath);
+			Xfree(origpath);
+			origpath = Xstrdup(fullpath);
 		}
 	}
-	strcpy2(fullpath, origpath);
+	Xstrcpy(fullpath, origpath);
 
 	tmp = NULL;
 #if	MSDOS
@@ -1244,14 +1307,14 @@ char *CONST envp[];
 #else
 	if (strdelim(argv, 0)) cp = argv;
 	else cp = tmp = searchexecname(argv, envp);
-	if (!cp) progpath = strdup2(BINDIR);
+	if (!cp) progpath = Xstrdup(BINDIR);
 	else
 #endif
 	{
-		realpath2(cp, buf, RLP_READLINK);
-		free2(tmp);
+		Xrealpath(cp, buf, RLP_READLINK);
+		Xfree(tmp);
 		if ((tmp = strrdelim(buf, 0))) *tmp = '\0';
-		progpath = strdup2(buf);
+		progpath = Xstrdup(buf);
 	}
 }
 
@@ -1274,7 +1337,7 @@ char *CONST *argv;
 		}
 #endif	/* !MSDOS */
 		for (i = 0; i < 2; i++) loadhistory(i);
-		entryhist(origpath, HST_PATH | HST_UNIQ);
+		VOID_C entryhist(origpath, HST_PATH | HST_UNIQ);
 	}
 #ifdef	DEP_LOGGING
 	startlog(argv);
@@ -1301,29 +1364,29 @@ int status;
 	if (origpath && _chdir2(origpath) < 0) {
 		perror2(origpath);
 		if (!Xgetwd(cwd)) *cwd = '\0';
-		rawchdir(rootpath);
+		VOID_C rawchdir(rootpath);
 	}
-	free2(origpath);
-	free2(progname);
+	Xfree(origpath);
+	Xfree(progname);
 #ifdef	DEP_UNICODE
-	free2(unitblpath);
+	Xfree(unitblpath);
 #endif
 #ifdef	DEP_IME
-	free2(dicttblpath);
+	Xfree(dicttblpath);
 #endif
 #ifndef	_NOCATALOG
-	free2(cattblpath);
+	Xfree(cattblpath);
 #endif
 #ifdef	DEP_DOSDRIVE
-	dosallclose();
+	VOID_C dosallclose();
 #endif
 #ifdef	DEP_URLPATH
-	urlallclose();
+	VOID_C urlallclose();
 #endif
-	free2(progpath);
+	Xfree(progpath);
 
 #ifdef	DEBUG
-	free2(tmpfilename);
+	Xfree(tmpfilename);
 	tmpfilename = NULL;
 # ifndef	DEP_ORIGSHELL
 	freevar(environ);
@@ -1335,17 +1398,17 @@ int status;
 #  endif
 #  ifndef	DEP_DYNAMICLIST
 	freestrarray(orighelpindex, MAXHELPINDEX);
-	free2(orighelpindex);
-	free2(origbindlist);
+	Xfree(orighelpindex);
+	Xfree(origbindlist);
 #   ifndef	_NOARCHIVE
 	freelaunchlist(origlaunchlist, origmaxlaunch);
-	free2(origlaunchlist);
+	Xfree(origlaunchlist);
 	freearchlist(origarchivelist, origmaxarchive);
-	free2(origarchivelist);
+	Xfree(origarchivelist);
 #   endif
 #   ifdef	DEP_DOSEMU
 	freedosdrive(origfdtype, origmaxfdtype);
-	free2(origfdtype);
+	Xfree(origfdtype);
 #   endif
 #  endif	/* !DEP_DYNAMICLIST */
 # endif	/* !_NOCUSTOMIZE */
@@ -1358,7 +1421,7 @@ int status;
 # endif
 	chdir2(NULL);
 # if	!defined (_NOUSEHASH) && !defined (DEP_ORIGSHELL)
-	searchhash(NULL, NULL, NULL);
+	VOID_C searchhash(NULL, NULL, NULL);
 # endif
 # ifndef	_NOROCKRIDGE
 	detranspath(NULL, NULL);
@@ -1375,7 +1438,7 @@ int status;
 #ifndef	_NOCATALOG
 	freecatalog();
 #endif
-	if (*cwd) rawchdir(cwd);
+	if (*cwd) VOID_C rawchdir(cwd);
 }
 
 int main(argc, argv, envp)
@@ -1385,6 +1448,9 @@ char *CONST argv[], *CONST envp[];
 	char *cp;
 	int i;
 
+#ifndef	NOFLOCK
+	stream_isnfsfunc = isnfs;
+#endif
 #ifdef	DEBUG
 	mtrace();
 #endif
@@ -1398,56 +1464,11 @@ char *CONST argv[], *CONST envp[];
 	_harderr(criticalerror);
 #endif
 
-#ifdef	SIGHUP
-	signal2(SIGHUP, (sigcst_t)hanguperror);
-#endif
-#ifdef	SIGINT
-	signal2(SIGINT, (sigcst_t)ignore_int);
-#endif
-#ifdef	SIGQUIT
-	signal2(SIGQUIT, (sigcst_t)ignore_quit);
-#endif
-#ifdef	SIGCONT
-	signal2(SIGCONT, (sigcst_t)ignore_cont);
-#endif
-#ifdef	SIGILL
-	signal2(SIGILL, (sigcst_t)illerror);
-#endif
-#ifdef	SIGTRAP
-	signal2(SIGTRAP, (sigcst_t)traperror);
-#endif
-#ifdef	SIGIOT
-	signal2(SIGIOT, (sigcst_t)ioerror);
-#endif
-#ifdef	SIGEMT
-	signal2(SIGEMT, (sigcst_t)emuerror);
-#endif
-#ifdef	SIGFPE
-	signal2(SIGFPE, (sigcst_t)floaterror);
-#endif
-#ifdef	SIGBUS
-	signal2(SIGBUS, (sigcst_t)buserror);
-#endif
-#ifdef	SIGSEGV
-	signal2(SIGSEGV, (sigcst_t)segerror);
-#endif
-#ifdef	SIGSYS
-	signal2(SIGSYS, (sigcst_t)syserror);
-#endif
-#ifdef	SIGPIPE
-	signal2(SIGPIPE, (sigcst_t)pipeerror);
-#endif
-#ifdef	SIGTERM
-	signal2(SIGTERM, (sigcst_t)terminate);
-#endif
-#ifdef	SIGXCPU
-	signal2(SIGXCPU, (sigcst_t)xcpuerror);
-#endif
-#ifdef	SIGXFSZ
-	signal2(SIGXFSZ, (sigcst_t)xsizerror);
-#endif
+	for (i = 0; i < arraysize(sigfunclist); i++)
+		signal2(sigfunclist[i].sig, (sigcst_t)(sigfunclist[i].func));
 	sigvecset(0);
 
+	VOID_C entryhist(NULL, HST_INIT);
 #ifdef	DEP_DYNAMICLIST
 	helpindex = copystrarray(NULL, orighelpindex, NULL, MAXHELPINDEX);
 	for (i = 0; origbindlist[i].key >= 0; i++) /*EMPTY*/;
@@ -1489,32 +1510,32 @@ char *CONST argv[], *CONST envp[];
 
 #ifndef	DEP_ORIGSHELL
 	i = countvar(envp);
-	environ = (char **)malloc2((i + 1) * sizeof(char *));
-	for (i = 0; envp[i]; i++) environ[i] = strdup2(envp[i]);
+	environ = (char **)Xmalloc((i + 1) * sizeof(char *));
+	for (i = 0; envp[i]; i++) environ[i] = Xstrdup(envp[i]);
 	environ[i] = NULL;
 #endif
 
 	setexecpath(argv[0], envp);
 #ifdef	DEP_UNICODE
-	unitblpath = strdup2(DATADIR);
+	unitblpath = Xstrdup(DATADIR);
 #endif
 #ifdef	DEP_IME
-	dicttblpath = strdup2(DATADIR);
+	dicttblpath = Xstrdup(DATADIR);
 #endif
 #ifndef	_NOCATALOG
 # ifdef	USEDATADIR
 	cp = getversion(&i);
-	cattblpath = malloc2(strsize(DATADIR) + 1 + i + 1);
-	snprintf2(cattblpath, strsize(DATADIR) + 1 + i + 1,
+	cattblpath = Xmalloc(strsize(DATADIR) + 1 + i + 1);
+	VOID_C Xsnprintf(cattblpath, strsize(DATADIR) + 1 + i + 1,
 		"%s%c%-.*s", DATADIR, _SC_, i, cp);
 # else
-	cattblpath = strdup2(DATADIR);
+	cattblpath = Xstrdup(DATADIR);
 # endif
 #endif	/* !_NOCATALOG */
 
 #ifdef	DEP_ORIGSHELL
 	cp = getshellname(progname, NULL, NULL);
-	if (!strpathcmp(cp, FDSHELL) || !strpathcmp(cp, "su")) {
+	if (!strpathcmp(cp, FDSHELL) || !strpathcmp(cp, SWITCHUSER)) {
 		i = main_shell(argc, argv, envp);
 # ifdef	DEP_PTY
 		killallpty();
@@ -1523,7 +1544,7 @@ char *CONST argv[], *CONST envp[];
 	}
 	interactive = fdmode = 1;
 	setshellvar(envp);
-	prepareterm();
+	if (prepareterm() < 0) error(NULL);
 	argc = initoption(argc, argv);
 	if (dumbterm > 1) {
 		errno = 0;
@@ -1570,11 +1591,7 @@ char *CONST argv[], *CONST envp[];
 #ifdef	SIGWINCH
 	nowinch = 0;
 #endif
-#if	FD >= 2
-	main_fd(&(argv[i]), (thruargs) ? 1 : 0);
-#else
 	main_fd(&(argv[i]), 0);
-#endif
 	sigvecset(0);
 
 	Xstdiomode();
